import datetime
import os
import shutil
import socket
import time

from django.template import loader
import xml.etree.ElementTree as ET

from .models import Track, SomaLogLine, StreamedDiffusion, Jingle, LOCAL_BASE_PATH


class SomaException(Exception):
    pass


def get_current_nonstop_track():
    try:
        soma_log_line = SomaLogLine.objects.select_related().order_by('-play_timestamp')[0]
    except IndexError:
        # nothing yet
        return {}
    if soma_log_line.play_timestamp < (datetime.datetime.now() - datetime.timedelta(hours=1)):
        # last known line is way too old
        return {}
    if not soma_log_line.on_air:
        # nonstop should be on air but it's not :/
        return {}
    d = {}
    current_nonstop_file = soma_log_line.filepath
    if not 'Tranches/' in current_nonstop_file.filepath and (
            not 'tracks/' in current_nonstop_file.filepath):
        # nonstop is playing but it's not a nonstop track :/
        return {}
    current_track = soma_log_line.filepath.track
    if current_track is None:
        # nonstop is playing a nonstop track, but it's unknown :/
        return {}
    d = {'track_title': current_track.title}
    if current_track.artist:
        d['track_artist'] = current_track.artist.name
    return d


def get_diffusion_file_path(diffusion):
    return u'diffusions-auto/%s--%s' % (
            diffusion.datetime.strftime('%Y%m%d-%H%M'),
            diffusion.episode.emission.slug)

def is_already_in_soma(diffusion):
    if StreamedDiffusion.objects.filter(diffusion=diffusion).count():
        return True
    return os.path.exists(os.path.join(LOCAL_BASE_PATH, get_diffusion_file_path(diffusion)))

class DuplicateDiffusionSlot(Exception):
    pass


def soma_connection():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(('soma', 12521))
    s.recv(1024)  # -> b'soma 2.5 NO_SSL\n'
    s.sendall(b'100 - Ok\n')
    s.recv(1024)  # -> b'100 - Welcome to soma daemon\n'
    s.sendall(b'\n')  # (empty password)
    if s.recv(1024) != b'100 - Ok\n':
        raise SomaException('failed to initialize soma connection')
    return s


def add_diffusion(diffusion, **kwargs):
    context = {}
    streamed_diffusion = StreamedDiffusion.objects.filter(diffusion=diffusion).first()
    if streamed_diffusion:
        # program a stream
        context['streamed_diffusion'] = streamed_diffusion
        context['end'] = diffusion.end_datetime
        context['jingle'] = streamed_diffusion.jingle
    else:
        # program a soundfile
        soundfile = diffusion.episode.soundfile_set.filter(fragment=False)[0]
        diffusion_path = get_diffusion_file_path(diffusion)

        # copy file
        if not os.path.exists(LOCAL_BASE_PATH):
            raise SomaException('soma directory is not available')
        if os.path.exists(os.path.join(LOCAL_BASE_PATH, diffusion_path)):
            raise DuplicateDiffusionSlot()
        os.mkdir(os.path.join(LOCAL_BASE_PATH, diffusion_path))
        try:
            shutil.copyfile(soundfile.file.path,
                os.path.join(LOCAL_BASE_PATH, diffusion_path, os.path.basename(soundfile.file.path)))
        except IOError:
            try:
                os.rmdir(os.path.join(LOCAL_BASE_PATH, diffusion_path))
            except IOError:
                pass
            raise SomaException('error copying file to soma')

        context['diffusion_path'] = diffusion_path
        # end should be a bit before the real end of file so the same file doesn't
        # get repeated but shouldn't be less or equal than start date or soma would
        # loop the file
        context['end'] = diffusion.datetime + datetime.timedelta(seconds=
                max(((soundfile.duration or 480) - 180), 60))

        if kwargs.get('jingle_id'):
            context['jingle'] = Jingle.objects.get(id=kwargs['jingle_id'])

    # create palinsesti
    palinsesti_template = loader.get_template('nonstop/soma_palinsesti.xml')
    context['episode'] = diffusion.episode
    context['start'] = diffusion.datetime

    palinsesti = palinsesti_template.render(context)
    palinsesti_xml = ET.fromstring(palinsesti.encode('utf-8'))

    palinsesto_xml = get_palinsesto_xml()
    palinsesto_xml.append(palinsesti_xml)
    send_palinsesto_xml(palinsesto_xml)


def remove_streamed_diffusion(streamed_diffusion):
    palinsesto_xml = get_palinsesto_xml()
    stream_marker = '[stream:%s]' % streamed_diffusion.id
    for palinsesto in palinsesto_xml.findall('Palinsesto'):
        if palinsesto.findall('Description')[0].text.startswith(stream_marker):
            palinsesto_xml.remove(palinsesto)
    send_palinsesto_xml(palinsesto_xml)


def send_palinsesto_xml(palinsesto_xml):
    with soma_connection() as s:
        s.sendall(b'106 - Switch to a New Palinsesto Request\n')
        if s.recv(1024) != b'100 - Ok\n':
            raise SomaException('failed to switch palinsesto')
        s.sendall(ET.tostring(palinsesto_xml))

    # give it some time (...)
    time.sleep(3)
    with soma_connection() as s:
        s.sendall(b'122 - Set the current Palinsesto as Default\n')
        if s.recv(1024) != b'100 - Ok\n':
            raise SomaException('failed to set current palinsesto as default')


def get_palinsesto_xml():
    with soma_connection() as s:
        s.sendall(b'109 - Get the current palinsesto\n')
        palinsesto_bytes = b''
        while True:
            new_bytes = s.recv(200000)
            if not new_bytes:
                break
            palinsesto_bytes += new_bytes
        if not palinsesto_bytes.startswith(b'100 - Ok\n'):
            raise SomaException('failed to get palinsesto')
        palinsesto_bytes = palinsesto_bytes[9:]
    palinsesto_xml = ET.fromstring(palinsesto_bytes)
    return palinsesto_xml
